package net.printer.demo;

import com.sun.jna.Pointer;
import com.sun.jna.WString;
import com.sun.jna.ptr.IntByReference;
import com.sun.jna.ptr.PointerByReference;
import com.sun.jna.win32.StdCallLibrary;
import com.sun.jna.Native;

import java.io.IOException;
import java.io.UnsupportedEncodingException;

public class PrinterDemo {
    interface JnaCallBackDll extends StdCallLibrary {
        JnaCallBackDll instance = Native.load(getDllPath(), JnaCallBackDll.class);

        void PrinterCreator(PointerByReference handle, String model);

        int PortOpen(Pointer handle, String setting);

        int PrinterInitialize(Pointer handle);

        int PrintTextS(Pointer handle, byte[] data);

        int PrintAndFeedLine(Pointer handle);

        int PrintBarCode(Pointer handle, int bcType, String data, int width, int height, int alignment, int hriPosition);

        int CutPaperWithDistance(Pointer handle, int distance);

        int PrintSymbol(Pointer handle, int type, String data, int errLevel, int width, int height, int alignment);

        int SetRelativeHorizontal(Pointer handle, int position);

        int GetPrinterState(Pointer handle, IntByReference printerStatus);

        int OpenCashDrawer(Pointer handle, int pinMode, int onTime, int ofTime);

        int PrintImageW(Pointer handle, WString imagePathW, int scaleMode);

        int SetAlign(Pointer handle, int align);

        void ClosePort(Pointer handle);

        void ReleasePrinter(Pointer handle);

        void ListPrinters(byte[] buffer, int size, IntByReference needSize);

        void ListComPorts(byte[] buffer, int size, IntByReference needSize);
    }

    private static JnaCallBackDll ins;
    public static Pointer printer;

    public static JnaCallBackDll getIns() {
        if (ins == null) {
            ins = JnaCallBackDll.instance;
        }
        return ins;
    }

    private static String getDllPath() {
        try {
            if (System.getProperty("sun.arch.data.model").contains("64")) {
                return Native.extractFromResourcePath("printer.sdk.x64.dll").getAbsolutePath();
            } else {
                return Native.extractFromResourcePath("printer.sdk.x86.dll").getAbsolutePath();
            }
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }

    private static String parseStatus(int status) {
        if (0x12 == status) {
            return "Ready";
        } else if ((status & 0b100) > 0) {
            return "Cover opened";
        } else if ((status & 0b1000) > 0) {
            return "Feed button has been pressed";
        } else if ((status & 0b100000) > 0) {
            return "Printer is out of paper";
        } else if ((status & 0b1000000) > 0) {
            return "Error condition";
        } else {
            return "Error";
        }
    }

    public static String getStatus() {
        IntByReference status = new IntByReference(2);
        int rValue = ins.GetPrinterState(printer, status);
        if (rValue == 0) {
            return String.format("The printer status is %s", parseStatus(status.getValue()));
        } else {
            return String.format("Get Error, Code is: %d", rValue);
        }
    }

    public static void printSample() {
        ins.PrinterInitialize(printer);
        ins.SetRelativeHorizontal(printer, 180);
        ins.PrintTextS(printer, getString("Las vegas,NV5208\r\n"));
        ins.PrintAndFeedLine(printer);
        ins.PrintAndFeedLine(printer);
        ins.PrintTextS(printer, getString("Ticket #30-57320             User:HAPPY\r\n"));
        ins.PrintTextS(printer, getString("Station:52-102          Sales Rep HAPPY\r\n"));
        ins.PrintTextS(printer, getString("10/10/2019 3:55:01PM\r\n"));
        ins.PrintTextS(printer, getString("---------------------------------------\r\n"));
        ins.PrintTextS(printer, getString("Item         QTY         Price    Total\r\n"));
        ins.PrintTextS(printer, getString("Description\r\n"));
        ins.PrintTextS(printer, getString("---------------------------------------\r\n"));
        ins.PrintTextS(printer, getString("100328       1           7.99      7.99\r\n"));
        ins.PrintTextS(printer, getString("MAGARITA MIX 7           7.99      3.96\r\n"));
        ins.PrintTextS(printer, getString("680015       1          43.99     43.99\r\n"));
        ins.PrintTextS(printer, getString("LIME\r\n"));
        ins.PrintTextS(printer, getString("102501       1          43.99     43.99\r\n"));
        ins.PrintTextS(printer, getString("V0DKA\r\n"));
        ins.PrintTextS(printer, getString("021048       1           3.99      3.99\r\n"));
        ins.PrintTextS(printer, getString("ORANGE 3200Z\r\n"));
        ins.PrintTextS(printer, getString("---------------------------------------\r\n"));
        ins.PrintTextS(printer, getString("Subtobal                          60.93\r\n"));
        ins.PrintTextS(printer, getString("8.1% Sales Tax                     3.21\r\n"));
        ins.PrintTextS(printer, getString("2% Concession Recov                1.04\r\n"));
        ins.PrintTextS(printer, getString("---------------------------------------\r\n"));
        ins.PrintTextS(printer, getString("Total                             66.18\r\n"));
        ins.PrintBarCode(printer, 73, "1234567890", 3, 150, 0, 2);
        ins.CutPaperWithDistance(printer, 10);
        ins.OpenCashDrawer(printer, 0, 30, 255);
    }

    public static void printQRCode() {
        ins.PrinterInitialize(printer);
        ins.PrintTextS(printer, getString("Example qrcode.\r\n"));
        ins.PrintSymbol(printer, 49, "123456789", 48, 10, 10, 1);
        ins.SetAlign(printer, 0);
        ins.PrintTextS(printer, getString("Example PDF417.\r\n"));
        ins.PrintSymbol(printer, 48, "123456789", 48, 10, 8, 1);
        ins.SetAlign(printer, 0);
        ins.CutPaperWithDistance(printer, 10);
    }

    public static byte[] getString(String str) {
        try {
            byte[] org = str.getBytes("IBM437");
            byte[] ret = new byte[org.length + 1];
            System.arraycopy(org, 0, ret, 0, org.length);
            ret[org.length] = 0;
            return ret;
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }

        return new byte[1];
    }

    public static void printBarcode() {
        ins.PrinterInitialize(printer);
        ins.PrintTextS(printer, getString("Example UPC_A.\r\n"));
        ins.PrintBarCode(printer, 65, "614141999996", 3, 150, 0, 2);
        ins.PrintTextS(printer, getString("Example UPC_E.\r\n"));
        ins.PrintBarCode(printer, 66, "040100002931", 3, 150, 0, 2);
        ins.PrintTextS(printer, getString("Example JAN13(EAN13).\r\n"));
        ins.PrintBarCode(printer, 67, "2112345678917", 3, 150, 0, 2);
        ins.PrintTextS(printer, getString("Example JAN8(EAN8).\r\n"));
        ins.PrintBarCode(printer, 68, "21234569", 3, 150, 0, 2);
        ins.PrintTextS(printer, getString("Example CODE39.\r\n"));
        ins.PrintBarCode(printer, 69, "12345678", 3, 150, 0, 2);
        ins.PrintTextS(printer, getString("Example ITF.\r\n"));
        ins.PrintBarCode(printer, 70, "10614141999993", 3, 150, 0, 2);
        ins.PrintTextS(printer, getString("Example CODABAR.\r\n"));
        ins.PrintBarCode(printer, 71, "B1234567890B", 3, 150, 0, 2);
        ins.PrintTextS(printer, getString("Example CODE93.\r\n"));
        ins.PrintBarCode(printer, 72, "12345678", 3, 150, 0, 2);
        ins.PrintTextS(printer, getString("Example barcode 128.\r\n"));
        ins.PrintBarCode(printer, 73, "1234567890", 3, 150, 0, 2);
        ins.CutPaperWithDistance(printer, 10);
    }

    public static void testImage(String path) {
        ins.PrintImageW(printer, new WString(path), 0);
        ins.CutPaperWithDistance(printer, 10);
    }
}
