package net.printer.demo;

import com.sun.jna.Pointer;
import com.sun.jna.WString;
import com.sun.jna.ptr.IntByReference;
import com.sun.jna.ptr.PointerByReference;
import com.sun.jna.win32.StdCallLibrary;
import com.sun.jna.Native;

import java.io.IOException;
import java.io.UnsupportedEncodingException;

public class PrinterDemo {
    interface JnaCallBackDll extends StdCallLibrary {
        JnaCallBackDll instance = Native.load(getDllPath(), JnaCallBackDll.class);

        void PrinterCreator(PointerByReference handle, String model);

        int PortOpen(Pointer handle, String setting);

        int TSPL_Text(Pointer handle, int x, int y, String fontName, byte[] content, int rotation, int x_multiplication, int y_multiplication, int alignment);

        int TSPL_Print(Pointer handle, int num, int copies);

        int TSPL_Direction(Pointer handle, int direction, int mirror);

        int TSPL_Bar(Pointer handle, int x, int y, int width, int height);

        int TSPL_ImageW(Pointer handle, int x, int y, int mode, WString imgPathW);

        int TSPL_ClearBuffer(Pointer handle);

        int TSPL_Box(Pointer handle, int x, int y, int x_end, int y_end, int thickness, int radius);

        int TSPL_QrCode(Pointer handle, int x, int y, int width, int eccLevel, int mode, int rotate, int model, int mask, String data);

        int TSPL_Home(Pointer handle);
        int TSPL_BarCode(Pointer handle, int x, int y, int type, String content, int height, int showText, int rotation, int narrow, int wide);
        int TSPL_Setup(Pointer handle, int printSpeed, int printDensity, int labelWidth, int labelHeight, int labelType, int gapHight, int offset);

        int TSPL_GetPrinterStatus(Pointer handle, IntByReference printerStatus);

        int TSPL_PDF417(Pointer handle, int x, int y, int width, int height, int rotate, String option, String data);

        int TSPL_Block(Pointer handle, int x, int y, int width, int height, String fontName, String content, int rotation, int x_multiplication, int y_multiplication, int alignment);

        int TSPL_Dmatrix(Pointer handle, int x, int y, int width, int height, String content, int blockSize, int row, int col);

        void ClosePort(Pointer handle);

        void ReleasePrinter(Pointer handle);

        void ListPrinters(byte[] buffer, int size, IntByReference needSize);

        void ListComPorts(byte[] buffer, int size, IntByReference needSize);
    }

    private static JnaCallBackDll ins;
    public static Pointer printer;

    public static JnaCallBackDll getIns() {
        if (ins == null) {
            ins = JnaCallBackDll.instance;
        }
        return ins;
    }

    private static String getDllPath() {
        try {
            if (System.getProperty("sun.arch.data.model").contains("64")) {
                String path = Native.extractFromResourcePath("printer.sdk.x64.dll").getAbsolutePath();
                //System.out.println("path:"+path);
                return path;
            } else {
                return Native.extractFromResourcePath("printer.sdk.x86.dll").getAbsolutePath();
            }
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }

    private static String parseStatus(int status) {
        if (status == 0) {
            return "Normal!";
        } else if ((status & 0b1) > 0) {
            return "Head opened！";
        } else if ((status & 0b10) > 0) {
            return "Paper jam！";
        } else if ((status & 0b100) > 0) {
            return "Out of paper！";
        } else if ((status & 0b1000) > 0) {
            return "Out of ribbon！";
        } else if ((status & 0b10000) > 0) {
            return "Pause！";
        } else if ((status & 0b100000) > 0) {
            return "Printing！";
        } else if ((status & 0b1000000) > 0) {
            return "Cover opened！";
        } else {
            return "Other error！";
        }
    }

    public static String getStatus() {
        IntByReference status = new IntByReference(0);
        int rValue = ins.TSPL_GetPrinterStatus(printer, status);
        if (rValue == 0) {
            return String.format("The printer status is %s", parseStatus(status.getValue()));
        } else {
            return String.format("Get Error, Code is: %d", rValue);
        }
    }

    public static void printSample() {
        ins.TSPL_Setup(printer, 4, 8, 63, 101, 1, 2, 4);
        ins.TSPL_ClearBuffer(printer);
        ins.TSPL_Direction(printer, 0, 0);
        ins.TSPL_Box(printer, 6, 6, 384, 235, 5, 1);
        ins.TSPL_Box(printer, 16, 16, 376, 225, 5, 1);
        ins.TSPL_BarCode(printer, 30, 30, 7, "ABCDEFGH", 100, 0, 0, 2, 2);
        ins.TSPL_QrCode(printer, 265, 30, 4, 1, 0, 0, 1, 2, "test qrcode");
        ins.TSPL_Text(printer, 200, 144, "3", getString("Test EN"), 0, 1, 1, 0);
        ins.TSPL_Text(printer, 38, 165, "3", getString("Test EN"), 0, 1, 2, 0);
        ins.TSPL_Bar(printer, 200, 183, 166, 30);
        ins.TSPL_Bar(printer, 334, 145, 30, 30);
        ins.TSPL_Print(printer, 1, 1);
    }

    public static void printQRCode() {
        ins.TSPL_Setup(printer, 4, 8, 76, 30, 1, 0, 0);
        ins.TSPL_ClearBuffer(printer);
        ins.TSPL_QrCode(printer, 265, 30, 8, 1, 0, 0, 1, 2, "test qrcode");
        ins.TSPL_Print(printer, 1, 1);
    }

    public static byte[] getString(String str) {
        try {
            byte[] org = str.getBytes("IBM437");
            byte[] ret = new byte[org.length + 1];
            System.arraycopy(org, 0, ret, 0, org.length);
            ret[org.length] = 0;
            return ret;
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }

        return new byte[1];
    }

    public static void printBarcode() {
        ins.TSPL_Setup(printer, 4, 8, 76, 30, 1, 0, 0);
        ins.TSPL_ClearBuffer(printer);
        ins.TSPL_BarCode(printer, 30, 30, 7, "ABCDEFGH", 100, 2, 0, 2, 2);
        ins.TSPL_Print(printer, 1, 1);
    }

    public static void testImage(String path) {
        ins.TSPL_Setup(printer, 4, 8, 76, 80, 1, 2, 0);
        ins.TSPL_ClearBuffer(printer);
        ins.TSPL_ImageW(printer, 10, 50, 0, new WString(path));
        ins.TSPL_Print(printer, 1, 1);
    }
}
