/*
 * Created by JFormDesigner on Mon Apr 29 11:05:02 CST 2024
 */

package net.printer.demo;

import com.sun.jna.Pointer;
import com.sun.jna.ptr.IntByReference;
import com.sun.jna.ptr.PointerByReference;

import java.awt.*;
import java.awt.event.*;
import java.io.File;
import java.nio.charset.StandardCharsets;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import javax.swing.*;
import javax.swing.filechooser.FileNameExtensionFilter;

/**
 * @author star
 */
public class SampleFrame extends JFrame {
    private PrinterDemo.JnaCallBackDll ins;
    private SimpleDateFormat formatter= new SimpleDateFormat("HH:mm:ss: ");
    public SampleFrame() {
        initComponents();
        tipsTa.setLineWrap(true);
        tipsTa.setWrapStyleWord(true);
        ins = PrinterDemo.getIns();
        lptCb.addItem("LPT1");
        lptCb.addItem("LPT2");
        lptCb.addItem("LPT3");
        baudrateCb.addItem("9600");
        baudrateCb.addItem("19200");
        baudrateCb.addItem("38400");
        baudrateCb.addItem("57600");
        baudrateCb.addItem("115200");
        List<String> usbPaths = getDeviceList(true);
        for (String usbPath : usbPaths) {
            usbCb.addItem(usbPath);
        }
        List<String> serialPaths = getDeviceList(false);
        for (String serialPath : serialPaths) {
            portCb.addItem(serialPath);
        }
    }
    private List<String> getDeviceList(boolean isUsb) {
        ArrayList<String> ret = new ArrayList<>();
        int modeSize = System.getProperty("os.name").contains("Windows") ? 64 : 32;
        int portSize = System.getProperty("os.name").contains("Windows") ? 520 : 260;
        int allSize = modeSize + portSize;
        IntByReference needSize = new IntByReference(0);
        if (isUsb) {
            ins.ListPrinters(null, 0, needSize);
        } else {
            ins.ListComPorts(null, 0, needSize);
        }
        if (needSize.getValue() > 0) {
            byte[] buffer = new byte[needSize.getValue()];
            if (isUsb) {
                ins.ListPrinters(buffer, needSize.getValue(), needSize);
            } else {
                ins.ListComPorts(buffer, needSize.getValue(), needSize);
            }
            for (int i = 0; i < needSize.getValue(); i += allSize) {
                byte[] portPath = new byte[portSize];
                System.arraycopy(buffer, i + modeSize, portPath, 0, portPath.length);
                ret.add(new String(portPath, StandardCharsets.UTF_16LE).trim());
            }
        }
        return ret;
    }

    private void sampleClick(ActionEvent e) {
        addMsg("Print Sample");
        PrinterDemo.printSample();
    }

    private void barcodeClick(ActionEvent e) {
        addMsg("Barcode");
        PrinterDemo.printBarcode();
    }

    private void qrcodeClick(ActionEvent e) {
        addMsg("Qrcode");
        PrinterDemo.printQRCode();
    }

    private void imageClick(ActionEvent e) {
        openImgBrowser();
    }

    private void statusClick(ActionEvent e) {
        addMsg(PrinterDemo.getStatus());
    }

    private void OpenClick(ActionEvent e) {
        PointerByReference p = new PointerByReference(Pointer.NULL);
        ins.PrinterCreator(p, "");
        String info;
        if (usbRb.isSelected()){
            info = "USB,";
            if (usbCb.getSelectedItem() != null){
                info += usbCb.getSelectedItem();
            }
        }else if (comRb.isSelected()){
            info = portCb.getSelectedItem() + "," +  baudrateCb.getSelectedItem();
        }else if (netRb.isSelected()){
            info = "NET,"+ipTf.getText();
        } else {
            info = lptCb.getSelectedItem().toString();
        }
        if (ins.PortOpen(p.getValue(), info) == 0) {
            PrinterDemo.printer = p.getValue();
            refreshBtn(true);
            addMsg("Open port succeed!");
        } else {
            addMsg("Open port fail, please check!");
        }
    }

    private void closeClick(ActionEvent e) {
        if (PrinterDemo.printer != null){
            refreshBtn(false);
            addMsg("Close port");
            ins.ClosePort(PrinterDemo.printer);
            PrinterDemo.printer = null;
        }
    }

    private void refreshBtn(boolean isConnect){
        closeBtn.setEnabled(isConnect);
        openBtn.setEnabled(!isConnect);
        sampleBtn.setEnabled(isConnect);
        barcodeBtn.setEnabled(isConnect);
        qrcodeBtn.setEnabled(isConnect);
        imageBtn.setEnabled(isConnect);
        statusBtn.setEnabled(isConnect);
    }

    private void addMsg(String msg){
        tipsTa.append(formatter.format(new Date()) + msg + "\n");
        if (tipsTa.getLineCount() >= 30) {
            String txt = tipsTa.getText();
            tipsTa.setText(txt.substring(txt.length()/4));
        }
        JScrollBar jscrollBar = scrollPane2.getVerticalScrollBar();
        jscrollBar.setValue(jscrollBar.getMaximum());
    }

    private void openImgBrowser() {
        JFileChooser fileChooser = new JFileChooser();
        fileChooser.setDialogTitle("Select XML File");
        fileChooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
        fileChooser.setAcceptAllFileFilterUsed(false);
        fileChooser.addChoosableFileFilter(new FileNameExtensionFilter("Image Files", "jpg", "jpeg", "png", "bmp"));

        int result = fileChooser.showOpenDialog(null);
        if (result == JFileChooser.APPROVE_OPTION) {
            File selectedFile = fileChooser.getSelectedFile();
            addMsg("Print Image");
            PrinterDemo.testImage(selectedFile.getAbsolutePath());
        }

    }

    private void initComponents() {
        // JFormDesigner - Component initialization - DO NOT MODIFY  //GEN-BEGIN:initComponents  @formatter:off
        vSpacer2 = new JPanel(null);
        hSpacer4 = new JPanel(null);
        hSpacer5 = new JPanel(null);
        hSpacer7 = new JPanel(null);
        sampleBtn = new JButton();
        barcodeBtn = new JButton();
        usbRb = new JRadioButton();
        usbCb = new JComboBox<>();
        hSpacer3 = new JPanel(null);
        hSpacer6 = new JPanel(null);
        qrcodeBtn = new JButton();
        imageBtn = new JButton();
        comRb = new JRadioButton();
        portCb = new JComboBox<>();
        baudrateCb = new JComboBox<>();
        statusBtn = new JButton();
        netRb = new JRadioButton();
        ipTf = new JTextField();
        lptRb = new JRadioButton();
        lptCb = new JComboBox<>();
        scrollPane2 = new JScrollPane();
        tipsTa = new JTextArea();
        vSpacer6 = new JPanel(null);
        hSpacer2 = new JPanel(null);
        openBtn = new JButton();
        closeBtn = new JButton();

        //======== this ========
        setTitle("SDK Demo");
        setPreferredSize(new Dimension(600, 400));
        Container contentPane = getContentPane();
        contentPane.setLayout(new GridBagLayout());
        ((GridBagLayout)contentPane.getLayout()).columnWidths = new int[] {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
        ((GridBagLayout)contentPane.getLayout()).rowHeights = new int[] {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
        ((GridBagLayout)contentPane.getLayout()).columnWeights = new double[] {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0E-4};
        ((GridBagLayout)contentPane.getLayout()).rowWeights = new double[] {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0E-4};

        //---- vSpacer2 ----
        vSpacer2.setPreferredSize(new Dimension(10, 20));
        vSpacer2.setMinimumSize(new Dimension(12, 20));
        contentPane.add(vSpacer2, new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0,
            GridBagConstraints.CENTER, GridBagConstraints.BOTH,
            new Insets(0, 0, 5, 5), 0, 0));

        //---- hSpacer4 ----
        hSpacer4.setPreferredSize(new Dimension(20, 10));
        contentPane.add(hSpacer4, new GridBagConstraints(3, 0, 1, 1, 0.0, 0.0,
            GridBagConstraints.CENTER, GridBagConstraints.BOTH,
            new Insets(0, 0, 5, 5), 0, 0));

        //---- hSpacer5 ----
        hSpacer5.setPreferredSize(new Dimension(15, 10));
        hSpacer5.setMinimumSize(new Dimension(15, 12));
        contentPane.add(hSpacer5, new GridBagConstraints(5, 0, 1, 1, 0.0, 0.0,
            GridBagConstraints.CENTER, GridBagConstraints.BOTH,
            new Insets(0, 0, 5, 5), 0, 0));

        //---- hSpacer7 ----
        hSpacer7.setPreferredSize(new Dimension(20, 10));
        contentPane.add(hSpacer7, new GridBagConstraints(6, 0, 1, 1, 0.0, 0.0,
            GridBagConstraints.CENTER, GridBagConstraints.BOTH,
            new Insets(0, 0, 5, 5), 0, 0));

        //---- sampleBtn ----
        sampleBtn.setText("Print Sample");
        sampleBtn.setEnabled(false);
        sampleBtn.addActionListener(e -> sampleClick(e));
        contentPane.add(sampleBtn, new GridBagConstraints(2, 1, 1, 1, 0.0, 0.0,
            GridBagConstraints.CENTER, GridBagConstraints.BOTH,
            new Insets(0, 0, 5, 5), 0, 0));

        //---- barcodeBtn ----
        barcodeBtn.setText("Barcode");
        barcodeBtn.setEnabled(false);
        barcodeBtn.addActionListener(e -> barcodeClick(e));
        contentPane.add(barcodeBtn, new GridBagConstraints(4, 1, 1, 1, 0.0, 0.0,
            GridBagConstraints.CENTER, GridBagConstraints.BOTH,
            new Insets(0, 0, 5, 5), 0, 0));

        //---- usbRb ----
        usbRb.setText("USB");
        usbRb.setSelected(true);
        contentPane.add(usbRb, new GridBagConstraints(7, 1, 1, 1, 0.0, 0.0,
            GridBagConstraints.CENTER, GridBagConstraints.BOTH,
            new Insets(0, 0, 5, 5), 0, 0));
        contentPane.add(usbCb, new GridBagConstraints(8, 1, 2, 1, 0.0, 0.0,
            GridBagConstraints.CENTER, GridBagConstraints.BOTH,
            new Insets(0, 0, 5, 5), 0, 0));

        //---- hSpacer3 ----
        hSpacer3.setPreferredSize(new Dimension(20, 10));
        contentPane.add(hSpacer3, new GridBagConstraints(0, 2, 1, 1, 0.0, 0.0,
            GridBagConstraints.CENTER, GridBagConstraints.BOTH,
            new Insets(0, 0, 5, 5), 0, 0));

        //---- hSpacer6 ----
        hSpacer6.setPreferredSize(new Dimension(15, 10));
        hSpacer6.setMinimumSize(new Dimension(15, 12));
        contentPane.add(hSpacer6, new GridBagConstraints(1, 2, 1, 1, 0.0, 0.0,
            GridBagConstraints.CENTER, GridBagConstraints.BOTH,
            new Insets(0, 0, 5, 5), 0, 0));

        //---- qrcodeBtn ----
        qrcodeBtn.setText("Qrcode");
        qrcodeBtn.setEnabled(false);
        qrcodeBtn.addActionListener(e -> qrcodeClick(e));
        contentPane.add(qrcodeBtn, new GridBagConstraints(2, 2, 1, 1, 0.0, 0.0,
            GridBagConstraints.CENTER, GridBagConstraints.BOTH,
            new Insets(0, 0, 5, 5), 0, 0));

        //---- imageBtn ----
        imageBtn.setText("Print Image");
        imageBtn.setEnabled(false);
        imageBtn.addActionListener(e -> imageClick(e));
        contentPane.add(imageBtn, new GridBagConstraints(4, 2, 1, 1, 0.0, 0.0,
            GridBagConstraints.CENTER, GridBagConstraints.BOTH,
            new Insets(0, 0, 5, 5), 0, 0));

        //---- comRb ----
        comRb.setText("COM");
        contentPane.add(comRb, new GridBagConstraints(7, 2, 1, 1, 0.0, 0.0,
            GridBagConstraints.CENTER, GridBagConstraints.BOTH,
            new Insets(0, 0, 5, 5), 0, 0));

        //---- portCb ----
        portCb.setPreferredSize(new Dimension(90, 24));
        portCb.setMinimumSize(new Dimension(90, 24));
        contentPane.add(portCb, new GridBagConstraints(8, 2, 1, 1, 0.0, 0.0,
            GridBagConstraints.CENTER, GridBagConstraints.BOTH,
            new Insets(0, 0, 5, 5), 0, 0));

        //---- baudrateCb ----
        baudrateCb.setPreferredSize(new Dimension(90, 25));
        baudrateCb.setMinimumSize(new Dimension(90, 25));
        contentPane.add(baudrateCb, new GridBagConstraints(9, 2, 1, 1, 0.0, 0.0,
            GridBagConstraints.CENTER, GridBagConstraints.BOTH,
            new Insets(0, 0, 5, 5), 0, 0));

        //---- statusBtn ----
        statusBtn.setText("Printer Status");
        statusBtn.setEnabled(false);
        statusBtn.addActionListener(e -> statusClick(e));
        contentPane.add(statusBtn, new GridBagConstraints(2, 3, 1, 1, 0.0, 0.0,
            GridBagConstraints.CENTER, GridBagConstraints.BOTH,
            new Insets(0, 0, 5, 5), 0, 0));

        //---- netRb ----
        netRb.setText("NET");
        contentPane.add(netRb, new GridBagConstraints(7, 3, 1, 1, 0.0, 0.0,
            GridBagConstraints.CENTER, GridBagConstraints.BOTH,
            new Insets(0, 0, 5, 5), 0, 0));
        contentPane.add(ipTf, new GridBagConstraints(8, 3, 2, 1, 0.0, 0.0,
            GridBagConstraints.CENTER, GridBagConstraints.BOTH,
            new Insets(0, 0, 5, 5), 0, 0));

        //---- lptRb ----
        lptRb.setText("LPT");
        contentPane.add(lptRb, new GridBagConstraints(7, 4, 1, 1, 0.0, 0.0,
            GridBagConstraints.CENTER, GridBagConstraints.BOTH,
            new Insets(0, 0, 5, 5), 0, 0));
        contentPane.add(lptCb, new GridBagConstraints(8, 4, 2, 1, 0.0, 0.0,
            GridBagConstraints.CENTER, GridBagConstraints.BOTH,
            new Insets(0, 0, 5, 5), 0, 0));

        //======== scrollPane2 ========
        {
            scrollPane2.setMinimumSize(new Dimension(16, 205));
            scrollPane2.setPreferredSize(new Dimension(3, 205));

            //---- tipsTa ----
            tipsTa.setPreferredSize(null);
            scrollPane2.setViewportView(tipsTa);
        }
        contentPane.add(scrollPane2, new GridBagConstraints(1, 6, 5, 8, 0.0, 0.0,
            GridBagConstraints.CENTER, GridBagConstraints.BOTH,
            new Insets(0, 0, 5, 5), 0, 0));

        //---- vSpacer6 ----
        vSpacer6.setPreferredSize(new Dimension(10, 50));
        vSpacer6.setMinimumSize(new Dimension(12, 50));
        contentPane.add(vSpacer6, new GridBagConstraints(6, 8, 1, 1, 0.0, 0.0,
            GridBagConstraints.CENTER, GridBagConstraints.BOTH,
            new Insets(0, 0, 5, 5), 0, 0));
        contentPane.add(hSpacer2, new GridBagConstraints(10, 8, 1, 1, 0.0, 0.0,
            GridBagConstraints.CENTER, GridBagConstraints.BOTH,
            new Insets(0, 0, 5, 0), 0, 0));

        //---- openBtn ----
        openBtn.setText("Open");
        openBtn.addActionListener(e -> OpenClick(e));
        contentPane.add(openBtn, new GridBagConstraints(7, 9, 1, 1, 0.0, 0.0,
            GridBagConstraints.CENTER, GridBagConstraints.BOTH,
            new Insets(0, 0, 5, 5), 0, 0));

        //---- closeBtn ----
        closeBtn.setText("Close");
        closeBtn.setEnabled(false);
        closeBtn.addActionListener(e -> closeClick(e));
        contentPane.add(closeBtn, new GridBagConstraints(9, 9, 1, 1, 0.0, 0.0,
            GridBagConstraints.CENTER, GridBagConstraints.BOTH,
            new Insets(0, 0, 5, 5), 0, 0));
        pack();
        setLocationRelativeTo(getOwner());

        //---- buttonGroup1 ----
        ButtonGroup buttonGroup1 = new ButtonGroup();
        buttonGroup1.add(usbRb);
        buttonGroup1.add(comRb);
        buttonGroup1.add(netRb);
        buttonGroup1.add(lptRb);
        // JFormDesigner - End of component initialization  //GEN-END:initComponents  @formatter:on
    }

    // JFormDesigner - Variables declaration - DO NOT MODIFY  //GEN-BEGIN:variables  @formatter:off
    private JPanel vSpacer2;
    private JPanel hSpacer4;
    private JPanel hSpacer5;
    private JPanel hSpacer7;
    private JButton sampleBtn;
    private JButton barcodeBtn;
    private JRadioButton usbRb;
    private JComboBox<String> usbCb;
    private JPanel hSpacer3;
    private JPanel hSpacer6;
    private JButton qrcodeBtn;
    private JButton imageBtn;
    private JRadioButton comRb;
    private JComboBox<String> portCb;
    private JComboBox<String> baudrateCb;
    private JButton statusBtn;
    private JRadioButton netRb;
    private JTextField ipTf;
    private JRadioButton lptRb;
    private JComboBox<String> lptCb;
    private JScrollPane scrollPane2;
    private JTextArea tipsTa;
    private JPanel vSpacer6;
    private JPanel hSpacer2;
    private JButton openBtn;
    private JButton closeBtn;
    // JFormDesigner - End of variables declaration  //GEN-END:variables  @formatter:on
}
