﻿using System;
using System.Runtime.InteropServices;
using System.Text;

namespace WinSDKDemo
{
    public class PrinterDemo
    {
        [DllImport("printer.sdk.dll", CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Auto)]
        public static extern IntPtr PrinterCreator(ref IntPtr printer, string model);

        [DllImport("printer.sdk.dll", CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Auto)]
        public static extern int ReleasePrinter(IntPtr intPtr);

        [DllImport("printer.sdk.dll", CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Auto)]
        public static extern int OpenPort(IntPtr intPtr, string usb);
        [DllImport("printer.sdk.dll", CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Auto)]
        public static extern int ClosePort(IntPtr intPtr);
        [DllImport("printer.sdk.dll", CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Auto)]
        public static extern int WriteData(IntPtr intPtr, byte[] buffer, int size);

        [DllImport("printer.sdk.dll", CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Auto)]
        public static extern int ReadData(IntPtr intPtr, byte[] buffer, int size);
        [DllImport("printer.sdk.dll", CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Auto)]
        public static extern int CPCL_AddLabel(IntPtr intPtr, int offSet, int height, int qty);
        [DllImport("printer.sdk.dll", CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Auto)]
        public static extern int CPCL_SetAlign(IntPtr intPtr, int align);
        [DllImport("printer.sdk.dll", CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Auto)]
        public static extern int CPCL_AddInverseLine(IntPtr handle, int x, int y, int xEnd, int yEnd, int width);
        [DllImport("printer.sdk.dll", CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Auto)]
        public static extern int CPCL_AddText(IntPtr intPtr, int rotate, string fontName, int fontSize, int xPos, int yPos, string data);
        [DllImport("printer.sdk.dll", CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Auto)]
        public static extern int CPCL_AddBarCode(IntPtr intPtr, int rotate, int type, int width, int ratio, int height, int xPos, int yPos, string data);
        [DllImport("printer.sdk.dll", CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Auto)]
        public static extern int CPCL_AddBarCodeText(IntPtr intPtr, int enable, int fontType, int fontSize, int offset);
        [DllImport("printer.sdk.dll", CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Auto)]
        public static extern int CPCL_AddQRCode(IntPtr intPtr, int rotate, int xPos, int yPos, int model, int unitWidth, int eccLevel, string data);
        [DllImport("printer.sdk.dll", CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Auto)]
        public static extern int CPCL_AddPDF417(IntPtr intPtr, int rotate, int xPos, int yPos, int xDots, int yDots, int columns, int eccLevel, string data);
        [DllImport("printer.sdk.dll", CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Auto)]
        public static extern int CPCL_AddBox(IntPtr intPtr, int xPos, int yPos, int endXPos, int endYPos, int thickness);
        [DllImport("printer.sdk.dll", CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Auto)]
        public static extern int CPCL_AddLine(IntPtr intPtr, int xPos, int yPos, int endXPos, int endYPos, int thickness);
        [DllImport("printer.sdk.dll", CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Auto)]
        public static extern int CPCL_AddImage(IntPtr intPtr, int rotate, int xPos, int yPos, string filePath);

        [DllImport("printer.sdk.dll", CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Auto)]
        public static extern int CPCL_SetTextUnderline(IntPtr intPtr, int bunderlineold);
        [DllImport("printer.sdk.dll", CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Auto)]
        public static extern int CPCL_GetPrinterStatus(IntPtr intPtr, out int status);
        [DllImport("printer.sdk.dll", CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Auto)]
        public static extern int CPCL_Print(IntPtr intPtr);
        public delegate void MsgCallback(string message);

        private static string ParseStatus(int status)
        {
            if (status == 0)
            {
                return "Normal!";
            }
            else if ((status & 0b1) > 0)
            {
                return "Head opened!";
            }
            else if ((status & 0b10) > 0)
            {
                return "Paper jam!";
            }
            else if ((status & 0b100) > 0)
            {
                return "Out of paper!";
            }
            else if ((status & 0b1000) > 0)
            {
                return "Out of ribbon!";
            }
            else if ((status & 0b10000) > 0)
            {
                return "Pause!";
            }
            else if ((status & 0b100000) > 0)
            {
                return "Printing!";
            }
            else if ((status & 0b1000000) > 0)
            {
                return "Cover opened!";
            }
            else
            {
                return "Other error!";
            }
        }

        public static void GetStatus(IntPtr printer, MsgCallback callback)
        {
            int ret = PrinterDemo.CPCL_GetPrinterStatus(printer, out int status);
            if (ret == 0)
            {
                callback($"The printer status is {ParseStatus(status)}");
            }
            else
            {
                callback($"Get Error, Code is: {ret}");
            }
        }

        public static void PrintSample(IntPtr printer)
        {
            CPCL_AddLabel(printer, 0, 400, 1);
            CPCL_AddBox(printer, 10, 10, 510, 280, 5);
            CPCL_AddLine(printer, 10, 180, 510, 180, 4);
            CPCL_AddQRCode(printer, 0, 20, 20, 2, 6, 3, "QR CODE ABC123");
            CPCL_SetAlign(printer, 0);
            CPCL_AddText(printer, 0, "0", 0, 300, 20, "REVERSE");
            CPCL_AddInverseLine(printer, 290, 20, 390, 20, 24);
            CPCL_AddText(printer, 0, "0", 0, 170, 20, "FONT0");
            CPCL_AddText(printer, 0, "1", 0, 170, 60, "FONT1");
            CPCL_AddText(printer, 0, "2", 0, 170, 80, "FONT2");
            CPCL_AddText(printer, 0, "3", 0, 170, 120, "FONT3");
            CPCL_AddBarCode(printer, 0, 20, 1, 1, 50, 270, 80, "123456789");
            CPCL_AddText(printer, 0, "0", 0, 20, 210, "Hello World!");
            CPCL_AddLine(printer, 200, 200, 450, 200, 50);
            CPCL_Print(printer);
        }

        public static void PrintQRCode(IntPtr printer)
        {
            CPCL_AddLabel(printer, 0, 400, 1);
            CPCL_AddQRCode(printer, 0, 20, 20, 2, 8, 3, "QR CODE ABC123");
            CPCL_Print(printer);
        }

        public static void PrintBarCode(IntPtr printer)
        {
            CPCL_AddLabel(printer, 0, 800, 1);
            CPCL_AddBarCodeText(printer, 1, 7, 0, 0);
            CPCL_AddText(printer, 0, "0", 0, 0, 0, "Code 128");
            CPCL_AddBarCode(printer, 0, 20, 1, 1, 50, 0, 30, "123456789");
            CPCL_AddText(printer, 0, "0", 0, 0, 120, "UPC-E");
            CPCL_AddBarCode(printer, 0, 3, 1, 1, 50, 0, 150, "223456");
            CPCL_AddText(printer, 0, "0", 0, 0, 240, "EAN/JAN-13");
            CPCL_AddBarCode(printer, 0, 6, 1, 1, 50, 0, 270, "323456791234");
            CPCL_AddText(printer, 0, "0", 0, 0, 360, "Code 39");
            CPCL_AddBarCode(printer, 0, 12, 1, 1, 50, 0, 390, "72233445");
            CPCL_AddText(printer, 0, "0", 0, 250, 0, "UPC-A");
            CPCL_AddBarCode(printer, 0, 0, 1, 1, 50, 250, 30, "423456789012");
            CPCL_AddText(printer, 0, "0", 0, 250, 120, "EAN/JAN-8");
            CPCL_AddBarCode(printer, 0, 9, 1, 1, 50, 250, 150, "52233445");
            CPCL_AddText(printer, 0, "0", 0, 300, 360, "CODABAR");
            CPCL_AddBarCode(printer, 1, 22, 1, 1, 50, 300, 540, "A67859B");
            CPCL_AddText(printer, 0, "0", 0, 0, 480, "Code 93/Ext.93");
            CPCL_AddBarCode(printer, 0, 16, 1, 1, 50, 0, 510, "823456789");
            CPCL_AddBarCodeText(printer, 0, 7, 0, 0);
            CPCL_Print(printer);
        }

        public static void PrintImage(IntPtr printer, string path)
        {
            CPCL_AddLabel(printer, 0, 600, 1);
            CPCL_SetAlign(printer, 0);
            CPCL_AddImage(printer, 0, 40, 0, path);
            CPCL_Print(printer);
        }
    }
}
